#===========================================================
# BBS.pm テストアプリケーション
#
our $VERSION = "0001.20210828.1659";		# 初版
#===========================================================
# 本プログラムを起動するとTCPのポート8888番で着信待機します。
# telnetクライアントで着信ポートに接続するとサーバのコンソールに
# メッセージと接続したノード番号が表示されます。
# キーを入力をすると入力した文字が接続する全てのノードに
# 送出し画面に表示されます。
# telnetクライアントを終了するとノードが切断し、
# サーバのコンソールにメッセージが表示されます。
# プログラムのあるパスに作成される.lockファイルを削除すると
# サーバが停止してプログラムが終了します。
#===========================================================
BEGIN {
	$::DEBUG = '';
	binmode STDIN, ':encoding(cp932)';
	binmode STDOUT, ':encoding(cp932)';
	binmode STDERR, ':encoding(cp932)';
	use FindBin;
	use lib "$FindBin::Bin/../lib";
};

package App;

use strict;
use warnings;
use utf8;

use BBS;                            # BBSのロード
my $bbs = new BBS;                  # オブジェクト作成

$bbs->{'send_buffer'} = {};         # 送信バッファ(オブジェクト内に作る)
#            ->{node} = "{data}";


## ( onConnect : 接続時処理 )
sub onconnect {
   my $self = shift;

   printf "\n(%d) 接続しました ", $self->from();
   print "\n接続ノード: [ ".join(',', $self->nodes())." ] ";
}

## ( onDisconnect : 切断時処理 )
sub ondisconnect {
   my $self = shift;

   printf "\n(%d) 切断しました ", $self->from();
   print "\n接続ノード: [ ".join(',', $self->nodes())." ] ";
}

## ( onRecv : データ受信 )
sub onrecv {
   my $self = shift;
   my $rcvdata = join('', @_);                       # イベントから受信データを取得

   my @tonodes = $self->nodes();                     # ノードリストを取得
   map { $self->send( $_, $rcvdata ) } @tonodes;     # 受信データを全ノードに送信
}

## ( onSend : データ送信 )
sub onsend {
   my $self = shift;
   my $to_node = shift;                                  # イベントから送信対象のノード番号を取得
   my $send_data = join('', @_);                         # イベントから送信データを取得

   $self->{'send_buffer'}->{$to_node} = $send_data;      # 対象ノードの送信バッファに送信データを保存
}

## ( Output : データ出力 )
sub output {
   my $self = shift;

   my $to_node = $self->from();                              # 送信対象のノード番号を取得
   my $send_data = $self->{'send_buffer'}->{$to_node};       # 対象ノードの送信バッファからデータを取得
   if ( defined($send_data) ) {                              # 送信データを取得したら
      $self->{'send_buffer'}->{$to_node} = undef;                # 送信バッファを消去
      return $send_data;                                         # 送信データを返り値に設定してルーチンを抜ける
   }
   return;                                                   # ルーチンを抜ける（送信データを取得しなかったら）
}

$bbs->setsyshandler('onConnect',    sub { $bbs->App::onconnect(@_) });
$bbs->setsyshandler('onDisconnect', sub { $bbs->App::ondisconnect(@_) });
$bbs->setsyshandler('onRecv',       sub { $bbs->App::onrecv(@_) });
$bbs->setsyshandler('onSend',       sub { $bbs->App::onsend(@_) });
$bbs->setsyshandler('Output',       sub { $bbs->App::output(@_) });

print "TCP8888で着信待機します";
$bbs->start(8888);
print "\n終了しました";

#================================================================
=pod
=encoding utf8
=head1 スクリプト名
chat.pl - BBS.pm テストアプリケーション
=head1 著者
naoit0 <https://www.naoit0.com/projects/bbs_pm/>
=cut
1;